
#   MIDAS Server for the NetVar service

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]      ;#  load default support procedures

global MIDAS_NetVarServer_Verbose
set MIDAS_NetVarServer_Verbose 1

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_RegisterAccessServer_Verbose
set MIDAS_RegisterAccessServer_Verbose 0

package require NetVarCmd


# Use namespaces to isolate your methods
namespace eval urn:NetVarServer {

global env
source [file join $env(MIDASBASE) TclHttpd tcl RegisterAccessServer.tcl]    ;#  load common procedures


    set RegisterIndex 1

#    NetVar register attributes
#
#    -n         = variable name
#    -t         = variable type   (ie int  -  integer variable)
#    -f         = format (for read of integer types)
#    -a         = array (boolean)
#    -i         = array index
#    -p         = access permission (rw (default) : ro)

    set NetVar_options {
          {n.arg "Undefined"  "name"}
          {t.arg "int"  "type"}
          {f.arg "x"  "format"}
          {a.arg 0  "array"}
          {i.arg 0  "index"}
          {p.arg "rw"  "permission"}
    }

    set NetVar_Updateoptions {
          {n.arg 0xa55a  "name"}
          {t.arg 0xa55a  "type"}
          {f.arg 0xa55a  "format"}
          {a.arg 0xa55a  "array"}
          {i.arg 0xa55a  "index"}
          {p.arg 0xa55a  "permission"}
    }

    set NetVar_Usage ": NetVar  -n ? -t ? -f x|d -a 0|1 -i ? -p rw|ro"

    proc ServerVerbose {option} {
        global MIDAS_NetVarServer_Verbose
        global MIDAS_RegisterAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose
        set MIDAS_SOAPDomainHandler_Verbose $option
        set MIDAS_RegisterAccessServer_Verbose $option
        set MIDAS_NetVarServer_Verbose $option
        return [rpcvar::rpcvar string "0 OK"]
    }

    proc DefineRegister {Register Class Attributes} {
        global MIDAS_NetVarServer_Verbose
        upvar RegisterIndex RegisterIndex
        upvar RegisterNames RegisterNames

        if {$MIDAS_NetVarServer_Verbose} \
             {puts "called method NetVarServer::DefineRegister with parameters $Register $Class $Attributes"}
             
#    Check Register name is not already in use

        if {[info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x20006 Register name already in use"]
        }

#    Check Class is valid

        switch $Class {
            generic -
            NetVar  {}

            default {
               return [rpcvar::rpcvar string "0x20007 Class name is invalid or not supported"]
            }
        }
        
#    create register

        switch $Class {
            generic -
            NetVar  {

                set RegisterNames($Register) $RegisterIndex

                upvar R$RegisterIndex R
                set R(class) NetVar

                upvar NetVar_options options

                foreach item $options {
                    set opt [first [split [first $item] .]]
                    set R($opt) [second $item]
                }

                incr RegisterIndex
            }
            default {
                return [rpcvar::rpcvar string "0x2000e Invalid request"]
            }
        }
        
#    apply attributes

        set response [do_WriteRegisterAttributes $Register $Attributes]

        return [rpcvar::rpcvar string $response]
    }

    proc do_ReadRegisterAttributes {Register} {

        global MIDAS_NetVarServer_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_NetVarServer_Verbose} \
             {puts "called method NetVarServer::ReadRegisterAttributes with parameter $Register"}

        switch $R(class) {
            NetVar {
                 set attributes "-n$R(n) -t$R(t) -f$R(f) -a$R(a) -i$R(i) -p$R(p)"
            }
            default {
                return "0x2000e Invalid request"
            }
        }

        return "0 $attributes"
    }
    
    proc do_WriteRegisterAttributes {Register Attributes} {
        upvar 2 sigName1 sigName1
        upvar 2 sigName2 sigName2
    
        global MIDAS_NetVarServer_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_NetVarServer_Verbose} \
             {puts "called method NetVarServer::WriteRegisterAttributes with parameters $Register $Attributes"}
     
        switch $R(class) {
            NetVar  {

                upvar 2 NetVar_options options
                upvar 2 NetVar_Usage usage

                array set params [::cmdline::getKnownOptions Attributes $options $usage]

                foreach m [array names params] {
                    switch $m {
                        n  {set R(n) $params(n)}
                        t  {
                           switch $params(t) {
                               function -
                               func   {set R(t) "function"}
                               float -
                               floa   {set R(t) "float"}
                               double -
                               doub   {set R(t) "double"}
                               int    {set R(t) "int"}
                               uint   {set R(t) "uint"}
                               long   {set R(t) "long"}
                               ulong  -
                               ulon   {set R(t) "ulong"}
                               short -
                               shor   {set R(t) "short"}
                               ushort -
                               usho   {set R(t) "ushort"}
                               char   {set R(t) "char"}
                               uchar  {set R(t) "uchar"}
                               string -
                               strin  {set R(t) "string"}
                               default {return "0x2000e Invalid request"}
                           }
                        }
                        p  {
                            switch $params(p) {
                                rw -
                                ro  {set R(p) $params(p)}
                                default {return "0x2000e Invalid request"}
                            }
                        }
                        a  {
                            switch $params(a) {
                                0 -
                                n -
                                no  {set R(a) 0}
                                1 -
                                y -
                                yes {set R(a) 1}
                                default {return "0x2000e Invalid request"}
                            }
                        }
                        i  {set R(i) $params(i)}
                        f  {
                            switch $params(f) {
                                x -
                                d  {set R(f) $params(f)}
                                default {return "0x2000e Invalid request"}
                            }
                        }
                        default {}
                    }
                }
            }
            default {
                return "0x2000e Invalid request"
            }
        }
        
        return "0 OK"
    }

    proc do_WriteRegister {Register Value} {

        global MIDAS_NetVarServer_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_NetVarServer_Verbose} \
             {puts "called method NetVarServer::WriteRegister with parameters $Register $Value"}

        switch $R(t) {
            float   {set rc [NetVarassignNetFloat $R(n) $R(i) 1 $Value]}
            double  {set rc [NetVarassignNetDouble $R(n) $R(i) 1 $Value]}
            int     {set rc [NetVarassignNetInt $R(n) $R(i) 1 $Value]}
            uint    {set rc [NetVarassignNetUInt $R(n) $R(i) 1 $Value]}
            long    {set rc [NetVarassignNetLong $R(n) $R(i) 1 $Value]}
            ulong   {set rc [NetVarassignNetULong $R(n) $R(i) 1 $Value]}
            short   {set rc [NetVarassignNetShort $R(n) $R(i) 1 $Value]}
            char    {set rc [NetVarassignNetChar $R(n) $R(i) 1 $Value]}
            uchar   {set rc [NetVarassignNetUChar $R(n) $R(i) 1 $Value]}
            string  {set rc [NetVarassignNetString $R(n) $R(i) 1 $Value]}
            default {
                return "0x2000e Invalid request"
            }
        }

        if {[first $rc] == 0} then {return "0 OK"} else {return "0x2000a Application detected error"}
    }

    proc do_ReadRegister {Register} {

        global MIDAS_NetVarServer_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_NetVarServer_Verbose} \
             {puts "called method NetVarServer::ReadRegister with parameters $Register"}

        switch $R(t) {
            float   {set rc [NetVarreadNetFloat $R(n) $R(i) 1]}
            double  {set rc [NetVarreadNetDouble $R(n) $R(i) 1]}
            int     {set rc [NetVarreadNetInt $R(n) $R(i) 1]}
            uint    {set rc [NetVarreadNetUInt $R(n) $R(i) 1]}
            long    {set rc [NetVarreadNetLong $R(n) $R(i) 1]}
            ulong   {set rc [NetVarreadNetULong $R(n) $R(i) 1]}
            short   {set rc [NetVarreadNetShort $R(n) $R(i) 1]}
            char    {set rc [NetVarreadNetChar $R(n) $R(i) 1]}
            uchar   {set rc [NetVarreadNetUChar $R(n) $R(i) 1]}
            string  {set rc [NetVarreadNetString $R(n) $R(i) 1]}
            default {
                return "0x2000e Invalid request"
            }
        }

        if {[first $rc] == 0} then {return "0 [tail $rc]"} else {return "0x2000a Application detected error"}
     }

    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export DefineRegister
     SOAP::export InquireRegisters
     SOAP::export ReadRegisterAttributes
     SOAP::export ReadRegistersAttributes
     SOAP::export DeleteRegister
     SOAP::export DeleteRegisters
     SOAP::export WriteRegisterAttributes
     SOAP::export WriteRegistersAttributes
     SOAP::export WriteRegister
     SOAP::export WriteRegisters
     SOAP::export InitialiseRegister
     SOAP::export InitialiseRegisters
     SOAP::export ReadRegister
     SOAP::export ReadRegisters
     
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /NetVarServer -namespace urn:NetVarServer -uri urn:NetVarServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/NetVarServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}



