
# Load the SOAP service support framework
package require SOAP::Domain

# Use namespaces to isolate your methods
namespace eval urn:TapeServer {

    global MIDASTapeServer
    if {[info exists env(TapeServer)]} then {set resource $env(TapeServer)} else {set resource tape}
    set MIDASTapeServer "/$resource"
    
    proc MountVolume {id volume_name} {
        global MIDASTapeServer
        InsertLog "called method MountVolume with id = $id; volume name = $volume_name"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' :volume '$volume_name" !mntvol)"} m]
        set status "$m"
        return [rpcvar::rpcvar int $status]
    }

    proc OpenFile {id access_mode label_type record_length block_length file_name} {
        global MIDASTapeServer
        InsertLog "called method OpenFile with id = $id; access_mode = $access_mode; label_type = $label_type; record length = $record_length; block length = $block_length; file name = $file_name"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' :amode $access_mode :labels $label_type :rlen $record_length :blen $block_length :file '$file_name' !open)"} m]
        set status "$m"
        return [rpcvar::rpcvar int $status]
    }

    proc InquireDeviceStatus {id} {
        global MIDASTapeServer
        InsertLog "called method InquireDeviceStatus with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' ?dstatus)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc CloseFile {id} {
        global MIDASTapeServer
        InsertLog "called method CloseFile with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' !close)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc DismountVolume {id} {
        global MIDASTapeServer
        InsertLog "called method DismountVolume with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' !disvol)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc DeallocateDevice {id} {
        global MIDASTapeServer
        InsertLog "called method DeallocateDevice with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' !free)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireStreamStatus {id} {
        global MIDASTapeServer
        InsertLog "called method InquireStreamStatus with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' ?fstatus)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InitialiseVolume {id volume_name label_type density_code current_volume_name} {
        global MIDASTapeServer
        InsertLog "called method InitialiseVolume with id = $id; label type = $label_type; density code = $density_code current_volume_name = $current_volume_name"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' :volume '$volume_name' :labels $label_type :density $density_code :oldvol '$current_volume_name' !invol)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc IdentifyVolume {id} {
        global MIDASTapeServer
        InsertLog "called method IdentifyVolume with id = $id"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' !idvol)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireAvailableDevices {} {
        global MIDASTapeServer
        InsertLog "called method InquireAvailableDevices"
        set z [catch {eg "(tape :path '$MIDASTapeServer' ?drives)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc AllocateDevice {id device_name} {
        global MIDASTapeServer
        InsertLog "called method AllocateDevice with id = $id; device_name = $device_name"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :alias '$id' :drive '$device_name' !claim)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc ClaimTapeServer {} {
        global MIDASTapeServer
        InsertLog "called method ClaimTapeServer"
        set z [catch {eg "(tape :path '$MIDASTapeServer' !claim)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc FreeTapeServer {} {
        global MIDASTapeServer
        InsertLog "called method FreeTapeServer"
        set z [catch {eg "(tape :path '$MIDASTapeServer'  !free)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SetTapeServerState {state} {
        global MIDASTapeServer
        InsertLog "called method SetTapeServerState with state = $state"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :state $state !state)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireTapeServerState {} {
        global MIDASTapeServer
        InsertLog "called method InquireTapeServerState"
        set z [catch {eg "(tape :path '$MIDASTapeServer' ?state)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc AssociateStream {stream ids} {
        global MIDASTapeServer
        InsertLog "called method AssociateStream with stream = $stream; ids = $ids"
        set z [catch {eg "(tape :path '$MIDASTapeServer' !clear)"} m]
        set i 0
        foreach item $ids {
            set z [catch {eg "(tape :path '$MIDASTapeServer' :idnum $i :assid '$item')"} m]
            incr i
        }
        set z [catch {eg "(tape :path '$MIDASTapeServer' :stream $stream :smode 2 !stream)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireStreamAssociation  {stream} {
        global MIDASTapeServer
        InsertLog "called method InquireStreamAssociation with stream = $stream"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :stream '$stream' ?stream)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireStreamState {stream} {
        global MIDASTapeServer
        InsertLog "called method InquireStreamState with stream = $stream"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :stream '$stream' ?strstate)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc SetRunNumber {n} {
        global MIDASTapeServer
        InsertLog "called method SetRunNumber with n = $n"
        set z [catch {eg "(tape :path '$MIDASTapeServer' :run $n !run)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }

    proc InquireAcqStatus {} {
        global MIDASTapeServer
        InsertLog "called method InquireAcqStatus"
        set z [catch {eg "(tape :path '$MIDASTapeServer' ?gstatus)"} m]
        set status "$m"
        return [rpcvar::rpcvar string $status]
    }


    # We have to publish the public methods...

     SOAP::export MountVolume 
     SOAP::export OpenFile
     SOAP::export InquireDeviceStatus 
     SOAP::export CloseFile 
     SOAP::export DismountVolume 
     SOAP::export DeallocateDevice
     SOAP::export InquireStreamStatus 
     SOAP::export InitialiseVolume
     SOAP::export IdentifyVolume
     SOAP::export InquireAvailableDevices
     SOAP::export AllocateDevice
     SOAP::export ClaimTapeServer
     SOAP::export FreeTapeServer
     SOAP::export SetTapeServerState
     SOAP::export InquireTapeServerState 
     SOAP::export AssociateStream
     SOAP::export InquireStreamAssociation 
     SOAP::export InquireStreamState
     SOAP::export SetRunNumber
     SOAP::export InquireAcqStatus

}

# register this service with tclhttpd
SOAP::Domain::register  -prefix  /TapeServer -namespace urn:TapeServer -uri urn:TapeServer
